/*
 * Translated from Andrew Tuman's Pascal sources.
 */

#ifndef __DATE_H
#define __DATE_H

typedef unsigned DayTy;
typedef unsigned MonthTy;
typedef unsigned YearTy;
typedef unsigned long JulTy;

typedef struct {
	unsigned short year;
	unsigned char month;
	unsigned char day;
} BDateTy;

#define DATE_BUFFER_LEN 11

class DateTy {
public:
/*
 *	USA      (mm-dd-yy)
 *	American (mm/dd/yy)
 *	British  (dd/mm/yy)
 *	German   (dd.mm.yy)
 *	Italian  (dd-mm-yy)
 *	Japanese (yy/mm/dd)
 *	ANSI     (yy.mm.dd)
 *	xBase    (YYYYMMDD)
 */
	enum Formats {
		AMERICAN = 0, USA = 1, ANSI = 2, JAPANESE = 3, BRITISH = 4,
		GERMAN = 5, ITALIAN = 6, XBASE = 7, PACKED = 8, DEFAULT = 9
	};

	DateTy();
	DateTy(YearTy, MonthTy, DayTy);
	DateTy(JulTy);
	DateTy(const char *, DateTy::Formats = DEFAULT);

	const char* c_str(DateTy::Formats = DEFAULT, bool century = true) const;

	int IsLeap() const;
	int WeekDay() const;
	DayTy Day() const;
	MonthTy Month() const;
	YearTy Year(bool century = true) const;
	DayTy DaysInMonth() const;

	DateTy FirstMonthDay() const;
	DateTy LastMonthDay() const;
	DateTy DecMonth(unsigned = 1) const;
	DateTy IncMonth(unsigned = 1) const;

	operator JulTy() const {
		return JulNum;
	}
	bool operator<(const DateTy& rhs) const {
		return JulNum < rhs.JulNum;
	}
	bool operator==(const DateTy& rhs) const {
		return JulNum == rhs.JulNum;
	}
	DateTy& operator++() {
		JulNum += 1;
		return *this;
	}
	DateTy& operator--() {
		JulNum -= 1;
		return *this;
	}
	DateTy& operator+=(int delta) {
		JulNum += delta;
		return *this;
	}
	DateTy& operator-=(int delta) {
		JulNum -= delta;
		return *this;
	}
	JulTy operator-(const DateTy& rhs) const {
		return JulNum - rhs.JulNum;
	}
	JulTy operator+(int rhs) const {
		return JulNum + rhs;
	}

	static int DayOfWeek(YearTy, MonthTy, DayTy);
	static int LeapYear(YearTy);
	static void PrintMode(Formats = DateTy::BRITISH);

protected:
	static char GetDelimiter();
	static void Encode(YearTy, MonthTy, DayTy, JulTy&);
	static void Decode(YearTy&, MonthTy&, DayTy&, const JulTy);
	static JulTy DaysToYear(YearTy);
	static DayTy DaysToDate(YearTy, MonthTy, DayTy);

	JulTy IncOneMonth() const;
	JulTy DecOneMonth() const;

	void d2h(const char* value) const;
	void h2d(const char* value) const;

private:
	JulTy JulNum;
	static Formats PrintOption;
	mutable char buffer[16];
};

#endif // __DATE_H
