/*
 * DBF/NTX reader.
 * Written by Boris Botstein.
 */

#include <windows.h>
#include <stdio.h>

#pragma hdrstop
#include "dbfntx.hpp"

#ifdef USE_NT_CALLS
ntx_file::ntx_page::ntx_page(HANDLE id, long _parent,
	const ntx_file::ntx_root& root)
#else
ntx_file::ntx_page::ntx_page(FILE* id, long _parent,
	const ntx_file::ntx_root& root)
#endif
	throw(runtime_error) :
	parent(_parent), index(0), entries(-1), item_size(0), page(0) {
	union page_image {
		unsigned short entries;
		unsigned char bytes[NTX_PAGE_SIZE];
		unsigned short shorts[NTX_PAGE_SIZE / 2];
	} data;

#ifdef USE_NT_CALLS
	DWORD dwBytesRead;
	if(!::ReadFile(id, &data, sizeof(page_image), &dwBytesRead, 0) ||
		dwBytesRead != sizeof(page_image))
#else
	if(::fread(&data, sizeof(page_image), 1, id) != 1)
#endif
		throw runtime_error(" - can't read page");

	if(data.entries > root.max_item)
		throw runtime_error(" - invalid entries value");
	if(data.entries) {
		entries = data.entries;
		item_size = root.item_size;
		page = new char[item_size * (entries + 1)];
		unsigned border = NTX_PAGE_SIZE - root.item_size;
		unsigned from = 0;
		for(register unsigned short i = 0; i <= entries; i++, from += item_size) {
			unsigned index = *(data.shorts + i + 1);
			if(index > border)
				throw runtime_error(" - invalid internal offset value");
			::memcpy(page + from, data.bytes + index, item_size);
		}
	}
}

ntx_file::ntx_item* _fastcall ntx_file::ntx_page::find(long offset)
	const const throw(runtime_error){
	if(entries == -1) return 0;
	ntx_item* item;
	for(register unsigned short i = 0; i <= get_entries(); i++) {
		item = operator[](i);
		if(item->page == offset) {
			index = i;
			return item;
		}
	}
	throw runtime_error(" - invalid page (3)");
}

int _fastcall ntx_file::ntx_page::find(const char* key, int key_len) const {
	if(entries == -1) return -1;
	int l = 0, r = get_entries();
	while(l < r) {
		register i = (l + r) >> 1;
		if(::memcmp(key, operator[]((unsigned short)i)->key, key_len) > 0)
			l = i + 1;
		else
			r = i;
	}
	if(get_entries() == l && !operator[]((unsigned short)l)->page)
		index = (unsigned short)(l - 1);
	else
		index = (unsigned short)l;
	return ::memcmp(key, operator[](index)->key, key_len);
}

