/*
 * DBF/NTX reader.
 * Written by Boris Botstein.
 */

#include <windows.h>
#include <stdio.h>

#pragma hdrstop
#include "dbfntx.hpp"

dbf_field dbf_file::get_field(int field_no) {
	dbf_field_list::const_iterator i = fields.find(field_no);
	if(i != fields.end()) return dbf_field(buffer, *(*i).second);
	char tmp[32];
	::sprintf(tmp, "%d", field_no);
	throw runtime_error(_string(name) + _string(" - invalid field ") +
		_string(tmp));
}

dbf_field dbf_file::get_field(const char* field_name) {
	dbf_field_list::const_iterator i = fields.begin();
	while(i != fields.end()) {
		if(::strcmpi((*i).second->name, field_name) == 0)
			return dbf_field(buffer, *(*i).second);
		i++;
	}
	throw runtime_error(_string(name) + _string(" - invalid field ") +
		_string(field_name));
}

dbf_field::dbf_field(char* buffer, const dbf_field_desc& desc) :
	size(desc.length), dec(desc.dec), offset(desc.offset),
	type(desc.type), ptr(buffer) {
}

dbf_field::dbf_field() : size(0), dec(0), type(0), offset(0), ptr(0) {
}

dbf_field::operator DateTy() const {
	if(ptr)
		switch(type) {
		case 'D':
			initialize();
			return DateTy(field, DateTy::XBASE);
		case 'C':
			if(size == 4) {
				initialize();
				return DateTy(field, DateTy::PACKED);
			}
		}
	return DateTy(JulTy(0l));
}

dbf_field::operator double() const {
	if(ptr) {
		switch(type) {
		case 'N':
		  	initialize();
			return ::atof(field);
		case 'C':
			if(size == 8) {
			  	initialize();
				double value;
				::memcpy(&value, field, sizeof(double));
				return value;
			}
		}
	}
	return 0.0;
}

dbf_field::operator char*() const {
	if(ptr) {
	  	initialize();
		return (char*)field;
	}
	return 0;
}

char* dbf_field::ansi() const {
	char *pointer = operator char*();
	if(pointer) {
		::OemToChar(pointer, pointer);
		return pointer;
	}
	return 0;
}

dbf_field::operator bool() const {
	if(ptr) {
	  	initialize();
		if('L' == type && 'T' == field[0]) return true;
	}
	return false;
}

dbf_field::operator int() const {
	if(ptr) {
		switch(type) {
		case 'N':
		  	initialize();
			return ::atoi(field);
		case 'C':
			if(size == 4) {
			  	initialize();
				return (int)int32_image(field);
			}
		}
	}
	return 0;
}

