#ifndef __STL_UTLS_HPP
#define __STL_UTLS_HPP

#define string _string
#include <map>
#include <list>
#include <set>
#include <stdexcep>
#undef string
using namespace std;

template < int dec, class value_type > class round_to {
private:
	long double p;

public:
	round_to() : p(::pow10l(dec + 1)) {}

	value_type operator()(const value_type& value) {
		long double mv = (value >= value_type(0.0)) ? 1e-10 : -1e-10;
		long double t = (p * value + 5.0l) / 10.0l + mv;
		long double f = ::floorl(t);
		return value_type((f / p) * 10.0l);
	}
};

extern round_to<2, double> Round;

template <class T>
int sign(const T& value) {
	return value == 0 ? 0 : (value < 0 ? -1 : 1);
}

// From Mummit's STL Guide:
template <class ForwardIterator>
void map_release(ForwardIterator first, ForwardIterator last) {
	while(first != last) {
		delete (*first).second;
		first++;
	}
}

// From Mummit's STL Guide:
template <class ForwardIterator>
void sequence_release(ForwardIterator first, ForwardIterator last) {
	while(first != last) {
		delete (*first);
		first++;
	}
}

char* new_str(const char* src);
char* dup_str(const char* src);
_string scrypt(const char* src, unsigned size, bool oem_to_ansi = true);

_string get_path(const char* env);

class int32_image {
private:
	char image[5];
	int value;

	void initialize() {
		::memset(image, 0, sizeof(image));
		value = 0;
	}

public:
	int32_image(const char* _image);
	int32_image(int _value);


	operator int() const { return value; }
	operator char*() const { return (char*)image; }
};

#endif // __STL_UTLS_HPP

